#include "Arduino.h"
#include <Wire.h>

// #include "utility/twi.h"

#include <HardwareSerial.h>

/*
 Parameter
 =========

 Low:  11100011  (default 860SPS)

 ???00011: SPS

 000 : 8SPS
 001 : 16SPS
 010 : 32SPS
 011 : 64SPS
 100 : 128SPS (default)
 101 : 250SPS
 110 : 475SPS
 111 : 860SPS


 High: 11000011  (default 2,048V )

 1???0011: Kanal (Multiplex)
 1100???1: Gain  Bit [11:9]  (Volt)

 000 : FS = ±6.144V
 001 : FS = ±4.096V
 010 : FS = ±2.048V (default)
 011 : FS = ±1.024V
 100 : FS = ±0.512V
 101 : FS = ±0.256V
 110 : FS = ±0.256V
 111 : FS = ±0.256V

 */

unsigned char i2cAdrSonde = 0x48;  // Sonde
unsigned char i2cAdrPoti = 0x49;  // Poti

//#define TAKT 16000000UL

void Init_UART(unsigned long baud) {

	unsigned int teiler;

	teiler = F_CPU / (2UL * baud) - 1;

	UBRR0L = teiler;
	UBRR0H = (teiler >> 8);

	UCSR0A |= (0 << U2X0);
	UCSR0B |= (1 << TXEN0) | (1 << RXEN0);

	UCSR0C |= (1 << UMSEL00) | (1 << UCSZ01) | (1 << UCSZ00);

	DDRD |= (1 << PD4);
}

unsigned char Read_UART(void) {
	while (!(UCSR0A & (1 << RXC0))) {
	}
	return UDR0;
}

void Write_UART(unsigned char c) {
	while (!(UCSR0A & (1 << UDRE0))) {
	}
	UDR0 = c;
}

// ==== Ende  UART - Funktionen =======

// ========= I²C - Funktionen =========

static void writeRegister(unsigned short value, unsigned char i2c_a) {
	Wire.beginTransmission(i2c_a);
	Wire.write(1);
	Wire.write(value >> 8);
	Wire.write(value % 256);
	Wire.endTransmission();
}

static unsigned short readRegister(unsigned char i2c_a) {
	Wire.beginTransmission(i2c_a);
	Wire.write(0);
	Wire.endTransmission();
	Wire.requestFrom(i2c_a, (unsigned char) 2);
	return ((Wire.read() << 8) | Wire.read());
}

unsigned short readADC(unsigned short config, unsigned char i2c_a) {
	writeRegister(config, i2c_a);
	return readRegister(i2c_a);
}

// ===== Ende   I²C - Funktionen =========

void setup(void) {
//	Init_UART(115200UL);
	Init_UART(9600UL);


	Wire.begin();
	Wire.setClock(400000);
}

typedef union {
	unsigned char buf[4];
	signed short Data[2];
} SerialOut;

typedef union {
	unsigned char buf[4];
	unsigned short Config[2];
} SerialIn;

SerialOut adc;
SerialIn serialIn;

void loop(void) {
//	while (Serial.available() > 3) {
//    Serial.readBytes(serialIn.buf, 4);
//    Serial.write(adc.buf, 4);

	serialIn.buf[0] = Read_UART();
	serialIn.buf[1] = Read_UART();
	serialIn.buf[2] = Read_UART();
	serialIn.buf[3] = Read_UART();

	serialIn.buf[0] = 0b11100011;
	serialIn.buf[1] = 0b11000011;
	serialIn.buf[2] = 0b11100011;
	serialIn.buf[3] = 0b11000011;

	Write_UART(adc.buf[0]);
	Write_UART(adc.buf[1]);
	Write_UART(adc.buf[2]);
	Write_UART(adc.buf[3]);

	adc.Data[0] = readADC(serialIn.Config[0], i2cAdrSonde);
	adc.Data[1] = readADC(serialIn.Config[1], i2cAdrPoti);
//	}
}


